import argparse
import bittensor
import sys

from celium_collateral_contracts.common import get_evm_key_associations


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("--netuid", type=int, required=True)
    parser.add_argument("--hotkey", required=True)
    parser.add_argument("--block", type=int, default=None)
    parser.add_argument("--network", default="finney")
    args = parser.parse_args()

    with bittensor.subtensor(network=args.network) as subtensor:
        # get the uid of the given hotkey from the metagraph
        metagraph = subtensor.metagraph(netuid=args.netuid, block=args.block)
        neurons = [neuron for neuron in metagraph.neurons if neuron.hotkey == args.hotkey]
        if not neurons:
            print(f"No neuron found for hotkey {args.hotkey} on netuid {args.netuid}", file=sys.stderr)
            sys.exit(1)
        uid = neurons[0].uid

        associations = get_evm_key_associations(subtensor, args.netuid, args.block)
        if uid not in associations:
            print(f"No association found for hotkey {args.hotkey} on netuid {args.netuid}", file=sys.stderr)
            sys.exit(1)

        print(f"EVM key address for neuron {args.hotkey}(uid={uid}) on netuid {args.netuid}: {associations[uid]}")


if __name__ == "__main__":
    main()



# This file makes the scripts folder a Python package.

from .verify_contract import *
from .slash_collateral import *
from .setup_evm import *
from .send_to_ss58_precompile import *
from .reclaim_collateral import *
from .list_contracts import *
from .get_reclaim_requests import *
from .get_collaterals import *
from .get_all_associations import *
from .generate_keypair import *
from .finalize_reclaim import *
from .deposit_collateral import *
from .deny_request import *
from .common import *
from .collateral_contract import *
from .associate_evm_key import *



#!/usr/bin/env python3
"""
Executor Collateral Query Tool

This script allows users to query the collateral amount for a specific miner and executor UUID
in a given smart contract. It connects to a blockchain network, validates
the provided addresses, and retrieves the collateral information.

The script will output the collateral amount in TAO (the native token).
"""
import argparse
import sys
from celium_collateral_contracts.common import (
    get_web3_connection,
    get_executor_collateral,
    validate_address_format,
    get_miner_address_of_executor
)

def main():
    parser = argparse.ArgumentParser(
        description="Query the collateral amount for a specific miner and executor UUID in a smart contract"
    )
    parser.add_argument(
        "--contract-address",
        required=True,
        help="The address of the smart contract"
    )
    parser.add_argument(
        "--executor-uuid",
        required=True,
        help="The UUID of the executor (as a string or hex)"
    )
    parser.add_argument(
        "--network",
        default="finney",
        help="The Subtensor Network to connect to.",
    )
    args = parser.parse_args()
    validate_address_format(args.contract_address)
    w3 = get_web3_connection(args.network)
    collateral = get_executor_collateral(w3, args.contract_address, args.executor_uuid)
    miner_address = get_miner_address_of_executor(w3, args.contract_address, args.executor_uuid)

    print(
        f"Collateral for miner {miner_address}, executor {args.executor_uuid}: {collateral} TAO"
    )

if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        print(f"Error: {str(e)}", file=sys.stderr)
        sys.exit(1) 


#!/usr/bin/env python3

"""
Collateral Slashing Script

This script allows trustees to slash collateral from miners who have violated
protocol rules. It handles the creation of slashing requests with associated
URLs for verification purposes.
"""

import sys
import asyncio
import argparse
from uuid import UUID
from celium_collateral_contracts.common import (
    load_contract_abi,
    get_web3_connection,
    get_account,
    validate_address_format,
    build_and_send_transaction,
    wait_for_receipt,
    calculate_md5_checksum,
    get_revert_reason,
)


class SlashCollateralError(Exception):
    """Custom exception for errors that occur during collateral slashing operations."""
    pass


async def slash_collateral(
    w3,
    account,
    contract_address,
    url,
    executor_uuid,
):
    """Slash collateral from a miner.

    Args:
        w3 (Web3): Web3 instance
        account: The account to use for the transaction
        contract_address (str): Address of the collateral contract
        url (str): URL containing information about the slash
        executor_uuid (str): Executor UUID for the slashing operation

    Returns:
        dict: Transaction receipt with slash event details

    Raises:
        Exception: If the transaction fails
    """
    contract_abi = load_contract_abi()
    contract = w3.eth.contract(address=contract_address, abi=contract_abi)

    # Calculate MD5 checksum if URL is valid
    md5_checksum = "0" * 32
    if url.startswith(("http://", "https://")):
        print("Calculating MD5 checksum of URL content...", file=sys.stderr)
        md5_checksum = calculate_md5_checksum(url)
        print(f"MD5 checksum: {md5_checksum}", file=sys.stderr)

    executor_uuid_bytes = UUID(executor_uuid).bytes

    tx_hash = build_and_send_transaction(
        w3,
        contract.functions.slashCollateral(
            executor_uuid_bytes,
            url,
            bytes.fromhex(md5_checksum),
        ),
        account,
        gas_limit=200000,  # Higher gas limit for this function
    )

    receipt = wait_for_receipt(w3, tx_hash)
    if receipt['status'] == 0:
        revert_reason = get_revert_reason(w3, tx_hash, receipt['blockNumber'])
        raise SlashCollateralError(f"Transaction failed for slashing collateral. Revert reason: {revert_reason}")
    slash_event = contract.events.Slashed().process_receipt(receipt)[0]

    return receipt, slash_event


async def main():
    parser = argparse.ArgumentParser(
        description="Slash collateral from a miner."
    )
    parser.add_argument(
        "--contract-address",
        required=True,
        help="Address of the collateral contract"
    )
    parser.add_argument(
        "--url",
        required=True,
        help="URL containing information about the slash"
    )
    parser.add_argument(
        "--executor-uuid",
        help="Executor UUID for the slashing operation"
    )
    parser.add_argument("--private-key", help="Private key of the account to use")
    parser.add_argument("--network", default="finney", help="The Subtensor Network to connect to.")

    args = parser.parse_args()

    validate_address_format(args.contract_address)

    w3 = get_web3_connection(args.network)
    account = get_account(args.private_key)

    try:
        receipt, event = await slash_collateral(
            w3,
            account,
            args.contract_address,
            args.url,
            args.executor_uuid
        )

        print(f"Successfully slashed for executor {args.executor_uuid}")
        print("Event details:")
        print(f"  Executor ID: {event['args']['executorId']}")
        print(f"  Miner address: {event['args']['miner']}")
        print(
            f"  Amount: "
            f"{w3.from_wei(event['args']['amount'], 'ether')} TAO",
        )
        print(f"  URL: {event['args']['url']}")
        print(
            f"  URL Content MD5: "
            f"{event['args']['urlContentMd5Checksum'].hex()}",
        )
        print(
            f"  Transaction hash: {receipt['transactionHash'].hex()}")
        print(f"  Block number: {receipt['blockNumber']}")
    except Exception as e:
        print(f"Error: {str(e)}", file=sys.stderr)


if __name__ == "__main__":
    asyncio.run(main())



#!/usr/bin/env python3

"""
Reclaim Request Denial Script

This script allows trustees to deny collateral reclaim requests. It requires
a URL that explains the reason for denial, which is stored on-chain for
transparency and accountability.
"""

import sys
import argparse
import asyncio
from celium_collateral_contracts.common import (
    load_contract_abi,
    get_web3_connection,
    get_account,
    validate_address_format,
    build_and_send_transaction,
    wait_for_receipt,
    calculate_md5_checksum,
    get_revert_reason,
)


class DenyReclaimRequestError(Exception):
    """Raised when denying a reclaim request fails."""
    pass


async def deny_reclaim_request(
        w3, account, reclaim_request_id, url, contract_address):
    """Deny a reclaim request on the contract.

    Args:
        w3: Web3 instance
        account: Account to use for the transaction
        reclaim_request_id: ID of the reclaim request to deny
        url: URL containing the reason for denial
        contract_address: Address of the contract

    Returns:
        tuple: (deny_event, receipt)
    """
    validate_address_format(contract_address)

    contract_abi = load_contract_abi()
    contract = w3.eth.contract(address=contract_address, abi=contract_abi)

    # Calculate MD5 checksum of the URL content
    md5_checksum = "0" * 32
    if url.startswith(("http://", "https://")):
        print("Calculating MD5 checksum of URL content...", file=sys.stderr)
        md5_checksum = calculate_md5_checksum(url)
        print(f"MD5 checksum: {md5_checksum}", file=sys.stderr)

    tx_hash = build_and_send_transaction(
        w3,
        contract.functions.denyReclaimRequest(
            reclaim_request_id, url, bytes.fromhex(md5_checksum)
        ),
        account,
        gas_limit=200000,
    )

    receipt = wait_for_receipt(w3, tx_hash)
    if receipt['status'] == 0:
        # Get revert reason for failed transaction
        revert_reason = get_revert_reason(w3, tx_hash, receipt['blockNumber'])
        raise DenyReclaimRequestError(
            f"Transaction failed for denying reclaim request {reclaim_request_id}. Revert reason: {revert_reason}"
        )
    deny_event = contract.events.Denied().process_receipt(receipt)[0]

    return deny_event, receipt


async def main():
    parser = argparse.ArgumentParser(
        description="Deny a reclaim request on the Collateral contract"
    )
    parser.add_argument(
        "--contract-address", required=True, help="Address of the deployed Collateral contract"
    )
    parser.add_argument(
        "--reclaim-request-id", required=True, type=int, help="ID of the reclaim request to deny"
    )
    parser.add_argument("--url", required=True, help="URL containing the reason for denial")
    parser.add_argument("--private-key", help="Private key of the account to use")
    parser.add_argument("--network", default="finney", help="The Subtensor Network to connect to.")
    args = parser.parse_args()

    w3 = get_web3_connection(args.network)
    account = get_account(args.private_key)

    deny_event, receipt = await deny_reclaim_request(
        w3=w3,
        account=account,
        reclaim_request_id=args.reclaim_request_id,
        url=args.url,
        contract_address=args.contract_address,
    )

    print(f"Successfully denied reclaim request {args.reclaim_request_id}")
    print("Event details:")
    print(f"  Reclaim ID: {deny_event['args']['reclaimRequestId']}")
    print(f"  URL: {deny_event['args']['url']}")
    print(
        f"  URL Content MD5: {deny_event['args']['urlContentMd5Checksum'].hex()}")
    print(f"  Transaction hash: {receipt['transactionHash'].hex()}")
    print(f"  Block number: {receipt['blockNumber']}")

if __name__ == "__main__":
    asyncio.run(main())


import bittensor
import bittensor_wallet
import eth_account
import eth_utils
import eth_keys.datatypes


def associate_evm_key(
    subtensor: bittensor.Subtensor,
    wallet: bittensor_wallet.Wallet,
    evm_private_key: str,
    netuid: int,
) -> tuple[bool, str]:
    """
    Associate an EVM key with a given wallet for a specific subnet.

    Args:
        subtensor (bittensor.Subtensor): The Subtensor object to use for the transaction.
        wallet (bittensor.wallet): The wallet object containing the hotkey for signing
            the transaction. The wallet.hotkey will be associated with the EVM key.
        evm_private_key (str): The private key corresponding to the EVM address, used
            for signing the message.
        netuid (int): The numerical identifier (UID) of the Subtensor network.
    """
    evm_address = eth_account.Account.from_key(evm_private_key).address
    eth_private_key = eth_keys.datatypes.PrivateKey(bytes.fromhex(evm_private_key))

    # subtensor encodes the u64 block number as little endian bytes before hashing
    # https://github.com/opentensor/subtensor/blob/6b86ebf30d3fb83f9d43ed4ce713c43204394e67/pallets/subtensor/src/tests/evm.rs#L44
    # https://github.com/paritytech/parity-scale-codec/blob/v3.6.12/src/codec.rs#L220
    # https://github.com/paritytech/parity-scale-codec/blob/v3.6.12/src/codec.rs#L1439
    block_number = subtensor.get_current_block()
    encoded_block_number = block_number.to_bytes(length=8, byteorder="little")
    hashed_block_number = eth_utils.keccak(encoded_block_number)

    hotkey_bytes: bytes = wallet.hotkey.public_key
    message = hotkey_bytes + hashed_block_number
    signature = eth_private_key.sign_msg(message)

    call = subtensor.substrate.compose_call(
        call_module="SubtensorModule",
        call_function="associate_evm_key",
        call_params={
            "netuid": netuid,
            "hotkey": wallet.hotkey.ss58_address,
            "evm_key": evm_address,
            "block_number": block_number,
            "signature": signature.to_bytes(),
        },
    )

    return subtensor.sign_and_send_extrinsic(
        call,
        wallet,
        wait_for_inclusion=True,
        wait_for_finalization=True,
    )



#!/usr/bin/env python3

import json
import os
from web3 import Web3

# Configuration (replace with your actual values)
RPC_URL = "YOUR_RPC_URL"
CONTRACT_ADDRESS = "YOUR_CONTRACT_ADDRESS"
ABI_FILE = "abi.json"

def get_contract(rpc_url, contract_address, abi_file):
    """Connects to the blockchain and gets the contract instance."""
    w3 = Web3(Web3.HTTPProvider(rpc_url))
    if not w3.is_connected():
        raise Exception(f"Failed to connect to web3 provider at {rpc_url}")

    with open(abi_file, "r") as f:
        abi = json.load(f)

    contract = w3.eth.contract(address=contract_address, abi=abi)
    return w3, contract

def backup_collateral_state(contract, output_file="collateral_backup.json"):
    """Reads and backs up the state of collateral-related mappings."""
    backup_data = {}

    # NOTE: Reading full mappings is not directly supported via RPC.
    # You typically need to know the keys (addresses, reclaimIds, executorUuids)
    # or iterate through relevant events to collect keys.
    # This is a basic example assuming you have a way to get the relevant keys.

    # Example: Querying a few known addresses (replace with actual logic to get keys)
    known_addresses = ["0x...", "0x..."] # Replace with logic to get miner addresses
    known_reclaim_ids = [1, 2] # Replace with logic to get reclaim IDs
    # known_executor_uuids = [...] # You would need to track these as well

    backup_data["collaterals"] = {}
    for address in known_addresses:
        try:
            collateral = contract.functions.collaterals(address).call()
            backup_data["collaterals"][address] = collateral
        except Exception as e:
            print(f"Could not read collateral for {address}: {e}")

    backup_data["collateralUnderPendingReclaims"] = {}
    for address in known_addresses:
         try:
            pending = contract.functions.collateralUnderPendingReclaims(address).call()
            backup_data["collateralUnderPendingReclaims"][address] = pending
         except Exception as e:
            print(f"Could not read pending collateral for {address}: {e}")


    backup_data["validatorOfMiner"] = {}
    for address in known_addresses:
         try:
            validator = contract.functions.validatorOfMiner(address).call()
            backup_data["validatorOfMiner"][address] = validator
         except Exception as e:
            print(f"Could not read validator for {address}: {e}")

    backup_data["reclaims"] = {}
    for reclaim_id in known_reclaim_ids:
        try:
            reclaim = contract.functions.reclaims(reclaim_id).call()
            # Reclaim struct might need conversion if it contains complex types
            backup_data["reclaims"][reclaim_id] = {
                "miner": reclaim[0],
                "amount": reclaim[1],
                "denyTimeout": reclaim[2],
                "executorUuid": bytes(reclaim[3]).hex() # Convert bytes16 to hex string
            }
        except Exception as e:
             # This will also catch cases where reclaim_id does not exist (amount == 0)
            # print(f"Could not read reclaim {reclaim_id}: {e}")
            pass # Ignore non-existent reclaims

    # Backing up collateralPerExecutor is more complex as it's a nested mapping.
    # You would need lists of both miner addresses and their associated executorUuids.
    # backup_data["collateralPerExecutor"] = {}
    # for miner_address in known_addresses:
    #     backup_data["collateralPerExecutor"][miner_address] = {}
    #     for executor_uuid in miners_known_executors.get(miner_address, []): # Need to know executors per miner
    #         try:
    #             collateral = contract.functions.collateralPerExecutor(miner_address, bytes.fromhex(executor_uuid)).call()
    #             backup_data["collateralPerExecutor"][miner_address][executor_uuid] = collateral
    #         except Exception as e:
    #              print(f"Could not read collateral per executor {executor_uuid} for {miner_address}: {e}")


    with open(output_file, "w") as f:
        json.dump(backup_data, f, indent=4)

    print(f"Backup saved to {output_file}")

if __name__ == "__main__":
    try:
        w3, contract = get_contract(RPC_URL, CONTRACT_ADDRESS, ABI_FILE)
        backup_collateral_state(contract)
    except Exception as e:
        print(f"An error occurred: {e}") 


import argparse
import json
import pathlib
import sys

import bittensor

from celium_collateral_contracts.subtensor import associate_evm_key


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--wallet-name", required=True)
    parser.add_argument("--wallet-hotkey", required=True)
    parser.add_argument("--wallet-path")
    parser.add_argument("--network", default="finney")
    parser.add_argument("--netuid", type=int, default=12)
    args = parser.parse_args()

    wallet = bittensor.Wallet(
        name=args.wallet_name,
        hotkey=args.wallet_hotkey,
        path=args.wallet_path,
    )

    keyfile = (
        pathlib.Path(wallet.path)
        .expanduser()
        .joinpath(wallet.name, "h160", wallet.hotkey_str)
        .resolve()
    )
    evm_private_key = json.loads(keyfile.read_text())["private_key"]

    with bittensor.subtensor(network=args.network) as subtensor:
        success, error = associate_evm_key(subtensor, wallet, evm_private_key, args.netuid)

    if not success:
        print(f"Unable to Associate EVM Key. {error}", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()
